Module: Webgen::ContentProcessor::Tags

Defined in:
lib/webgen/content_processor/tags.rb

Overview

Processes special webgen tags to provide dynamic content.

webgen tags are an easy way to add dynamically generated content to websites, for example menus or breadcrumb trails.

See Webgen::Tag and all the classes in its namespace for further information.

Class Method Summary collapse

Class Method Details

.call(context) ⇒ Object

Replace all webgen tags in the content of context with the rendered content.



17
18
19
20
21
22
23
24
25
# File 'lib/webgen/content_processor/tags.rb', line 17

def self.call(context)
  context.website.ext.tag.replace_tags(context.content) do |tag, params, body|
    context.website.logger.debug do
      "Replacing tag #{tag} with data #{params.inspect} and body '#{body}' in <#{context.ref_node}>"
    end
    context.website.ext.tag.call(tag, params, body, context)
  end
  context
end